Package: GSVA
Version: 2.6.3
Title: Gene Set Variation Analysis for Microarray and RNA-Seq Data
Authors@R: c(person("Robert", "Castelo", role=c("aut", "cre"),
                    comment=c(ORCID="0000-0003-2229-4508"),
                    email="robert.castelo@upf.edu"),
             person("Justin", "Guinney", role="aut", email="jguinney@gmail.com"),
             person("Alexey", "Sergushichev", role="ctb", email="alsergbox@gmail.com"),
             person("Pablo Sebastian", "Rodriguez", role="ctb", email="pablo.rodriguez.bio2@gmail.com"),
             person("Axel", "Klenk", role="ctb", email="axel.klenk@gmail.com"),
             person("Chan Zuckerberg Initiative (CZI)", role="fnd"),
             person("Spanish Ministry of Science, Innovation and Universities (MCIU)", role="fnd"))
Depends: R (>= 4.0.0)
Imports: methods, stats, utils, graphics, BiocGenerics, MatrixGenerics,
        S4Vectors, S4Arrays, HDF5Array, SparseArray, DelayedArray,
        IRanges, Biobase, SummarizedExperiment, GSEABase, Matrix (>=
        1.5-0), DelayedMatrixStats, BiocParallel, SingleCellExperiment,
        BiocSingular, SpatialExperiment, sparseMatrixStats, cli, memuse
Suggests: RUnit, BiocStyle, knitr, rmarkdown, limma, RColorBrewer,
        org.Hs.eg.db, genefilter, edgeR, GSVAdata, sva, TENxPBMCData,
        TENxVisiumData, scrapper, bluster, igraph, shiny,
        shinydashboard, ggplot2, data.table, plotly, future, promises,
        shinybusy, shinyjs
LinkingTo: cli
Description: Gene Set Variation Analysis (GSVA) is a non-parametric,
        unsupervised method for estimating variation of gene set
        enrichment through the samples of a expression data set. GSVA
        performs a change in coordinate systems, transforming the data
        from a gene by sample matrix to a gene-set by sample matrix,
        thereby allowing the evaluation of pathway enrichment for each
        sample. This new matrix of GSVA enrichment scores facilitates
        applying standard analytical methods like functional
        enrichment, survival analysis, clustering, CNV-pathway analysis
        or cross-tissue pathway analysis, in a pathway-centric manner.
License: Artistic-2.0
VignetteBuilder: knitr
URL: https://github.com/rcastelo/GSVA
BugReports: https://github.com/rcastelo/GSVA/issues
Encoding: UTF-8
biocViews: FunctionalGenomics, Microarray, RNASeq, Pathways,
        GeneSetEnrichment
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Config/pak/sysreqs: libmagick++-dev gsfonts libicu-dev libpng-dev
        libxml2-dev libssl-dev zlib1g-dev
Repository: Bioconductor 3.23
Date/Publication: 2026-07-14 16:18:13 UTC
RemoteUrl: https://github.com/bioc/GSVA
RemoteRef: RELEASE_3_23
RemoteSha: f209b338e28f6f6f819af7d7c2da47d12c4ebf9d
NeedsCompilation: yes
Packaged: 2026-07-14 21:49:28 UTC; root
Author: Robert Castelo [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2229-4508>),
  Justin Guinney [aut],
  Alexey Sergushichev [ctb],
  Pablo Sebastian Rodriguez [ctb],
  Axel Klenk [ctb],
  Chan Zuckerberg Initiative (CZI) [fnd],
  Spanish Ministry of Science, Innovation and Universities (MCIU) [fnd]
Maintainer: Robert Castelo <robert.castelo@upf.edu>
Built: R 4.6.1; aarch64-apple-darwin23; 2026-07-14 21:53:20 UTC; unix
