# Generated by roxygen2: do not edit by hand

export("geneSets<-")
export("gsvaAnnotation<-")
export(computeGeneSetsOverlap)
export(deduplicateGeneSets)
export(filterGeneSets)
export(geneIdsToGeneSetCollection)
export(geneSetSizes)
export(geneSets)
export(gsva)
export(gsvaAnnotation)
export(gsvaEnrichment)
export(gsvaParam)
export(gsvaRanks)
export(gsvaScores)
export(guessGeneIdType)
export(igsva)
export(loadHDF5GSVAranks)
export(plageParam)
export(readGMT)
export(saveHDF5GSVAranks)
export(spatCor)
export(ssgseaParam)
export(zscoreParam)
exportClasses(GsvaExprData)
exportClasses(GsvaGeneSets)
exportClasses(GsvaMethodParam)
exportClasses(gsvaParam)
exportClasses(gsvaRanksParam)
exportClasses(plageParam)
exportClasses(ssgseaParam)
exportClasses(zscoreParam)
exportMethods(computeGeneSetsOverlap)
exportMethods(filterGeneSets)
exportMethods(geneSetSizes)
exportMethods(geneSets)
exportMethods(gsva)
exportMethods(gsvaAnnotation)
exportMethods(gsvaEnrichment)
exportMethods(gsvaRanks)
exportMethods(gsvaScores)
exportMethods(spatCor)
import(methods)
importClassesFrom(Biobase,ExpressionSet)
importClassesFrom(DelayedArray,DelayedArray)
importClassesFrom(DelayedArray,DelayedMatrix)
importClassesFrom(GSEABase,GeneIdentifierType)
importClassesFrom(GSEABase,GeneSet)
importClassesFrom(GSEABase,GeneSetCollection)
importClassesFrom(HDF5Array,HDF5Array)
importClassesFrom(HDF5Array,HDF5Matrix)
importClassesFrom(IRanges,IRanges)
importClassesFrom(Matrix,dgCMatrix)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importClassesFrom(SparseArray,SVT_SparseMatrix)
importClassesFrom(SpatialExperiment,SpatialExperiment)
importClassesFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(Biobase,selectSome)
importFrom(BiocGenerics,"annotation<-")
importFrom(BiocGenerics,"type<-")
importFrom(BiocGenerics,annotation)
importFrom(BiocGenerics,cbind)
importFrom(BiocGenerics,rbind)
importFrom(BiocGenerics,type)
importFrom(BiocParallel,"bpprogressbar<-")
importFrom(BiocParallel,"bpstopOnError<-")
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpnworkers)
importFrom(BiocParallel,bpok)
importFrom(BiocParallel,bpprogressbar)
importFrom(BiocParallel,bptry)
importFrom(BiocSingular,runExactSVD)
importFrom(DelayedArray,blockApply)
importFrom(DelayedArray,blockReduce)
importFrom(DelayedArray,chunkGrid)
importFrom(DelayedArray,chunkdim)
importFrom(DelayedArray,close)
importFrom(DelayedArray,colAutoGrid)
importFrom(DelayedArray,colSums)
importFrom(DelayedArray,currentBlockId)
importFrom(DelayedArray,defaultAutoGrid)
importFrom(DelayedArray,getAutoBlockLength)
importFrom(DelayedArray,gridReduce)
importFrom(DelayedArray,read_block)
importFrom(DelayedArray,rowAutoGrid)
importFrom(DelayedArray,rowRanges)
importFrom(DelayedArray,seed)
importFrom(DelayedArray,setAutoBPPARAM)
importFrom(DelayedArray,t)
importFrom(DelayedArray,write_block)
importFrom(DelayedMatrixStats,rowSds)
importFrom(GSEABase,AnnoOrEntrezIdentifier)
importFrom(GSEABase,ENSEMBLIdentifier)
importFrom(GSEABase,EntrezIdentifier)
importFrom(GSEABase,NullCollection)
importFrom(GSEABase,NullIdentifier)
importFrom(GSEABase,SymbolIdentifier)
importFrom(GSEABase,geneIdType)
importFrom(GSEABase,geneIds)
importFrom(GSEABase,mapIdentifiers)
importFrom(HDF5Array,HDF5RealizationSink)
importFrom(HDF5Array,loadHDF5SummarizedExperiment)
importFrom(HDF5Array,saveHDF5SummarizedExperiment)
importFrom(HDF5Array,writeHDF5Array)
importFrom(IRanges,CharacterList)
importFrom(IRanges,IRanges)
importFrom(IRanges,IntegerList)
importFrom(IRanges,end)
importFrom(IRanges,match)
importFrom(IRanges,ranges)
importFrom(IRanges,start)
importFrom(IRanges,width)
importFrom(Matrix,colSums)
importFrom(Matrix,nnzero)
importFrom(MatrixGenerics,colRanks)
importFrom(MatrixGenerics,colSums)
importFrom(MatrixGenerics,rowMeans)
importFrom(MatrixGenerics,rowSds)
importFrom(S4Arrays,ArrayViewport)
importFrom(S4Arrays,DummyArrayGrid)
importFrom(S4Arrays,is_sparse)
importFrom(S4Arrays,read_block)
importFrom(S4Arrays,refdim)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,metadata)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SparseArray,nzcount)
importFrom(SparseArray,rowMeans)
importFrom(SparseArray,rowSds)
importFrom(SpatialExperiment,SpatialExperiment)
importFrom(SummarizedExperiment,"assay<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert_danger)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_alert_warning)
importFrom(cli,cli_progress_bar)
importFrom(cli,cli_progress_done)
importFrom(cli,cli_progress_update)
importFrom(graphics,abline)
importFrom(graphics,grid)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,segments)
importFrom(memuse,Sys.meminfo)
importFrom(memuse,howbig)
importFrom(methods,getClass)
importFrom(methods,is)
importFrom(methods,isClass)
importFrom(methods,new)
importFrom(sparseMatrixStats,colRanks)
importFrom(sparseMatrixStats,rowRanges)
importFrom(stats,dist)
importFrom(stats,ecdf)
importFrom(stats,na.omit)
importFrom(stats,pnorm)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,sd)
importFrom(utils,capture.output)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,installed.packages)
importFrom(utils,packageDescription)
importFrom(utils,tail)
importMethodsFrom(Biobase,experimentData)
importMethodsFrom(Biobase,exprs)
importMethodsFrom(Biobase,featureNames)
importMethodsFrom(Biobase,phenoData)
importMethodsFrom(BiocParallel,"bpprogressbar<-")
importMethodsFrom(BiocParallel,bplapply)
importMethodsFrom(GSEABase,GeneSet)
importMethodsFrom(GSEABase,GeneSetCollection)
importMethodsFrom(GSEABase,geneIds)
importMethodsFrom(GSEABase,incidence)
importMethodsFrom(S4Vectors,"metadata<-")
importMethodsFrom(S4Vectors,metadata)
importMethodsFrom(SpatialExperiment,imgData)
importMethodsFrom(SpatialExperiment,spatialCoords)
importMethodsFrom(SummarizedExperiment,assayNames)
importMethodsFrom(SummarizedExperiment,assays)
importMethodsFrom(SummarizedExperiment,colData)
importMethodsFrom(SummarizedExperiment,rowData)
useDynLib(GSVA, .registration=TRUE)
