% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSVA-pkg-defunct.R
\name{GSVA-pkg-defunct}
\alias{GSVA-pkg-defunct}
\title{Defunct functions in package \code{GSVA}.}
\description{
The functions listed below are defunct and will be removed
in the next release.
}
\keyword{internal}
