% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSVA-pkg-deprecated.R
\name{GSVA-pkg-deprecated}
\alias{GSVA-pkg-deprecated}
\title{Deprecated functions in package \code{GSVA}.}
\description{
The functions listed below are deprecated and will be defunct in
the near future. When possible, alternative functions with similar
functionality are also mentioned.
}
\keyword{internal}
