% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R, R/GsvaMethodParam.R
\docType{class}
\name{GsvaMethodParam-class}
\alias{GsvaMethodParam-class}
\title{\code{GsvaMethodParam} class}
\description{
Virtual superclass of method parameter classes supported by \code{GSVA}.

A virtual superclass of the \code{GSVA} packages' method-specific
parameter classes.
}
\details{
\code{GSVA} implements four single-sample gene set analysis methods: PLAGE,
combined z-scores, ssGSEA, and GSVA.  All of them take at least an expression
data matrix and one or more gene sets as input.  Further common parameters
include an assay name for use with multi-assay expression data containers,
the gene ID type used by the expression data set, and a minimum and maximum
size for gene sets to limit the range of gene set sizes used in an analysis.
This virtual class provides the necessary slots for this shared parameter set
and serves as the parent class for all \code{GSVA} method parameter classes.

The \code{GSVA} package implements four single-sample gene set analysis
methods (PLAGE, combined z-scores, ssGSEA, and GSVA) and a respective
method-specific parameter class that is used to invoke each of them with a
matching set of parameters.
}
\section{Slots}{

\describe{
\item{\code{exprData}}{The expression data set.  Must be one of the classes
supported by \code{GsvaExprData}.  For a list of these classes, see its
help page using \code{help(GsvaExprData)}.}

\item{\code{geneSets}}{The gene sets.  Must be one of the classes supported by
\code{GsvaGeneSets}.  For a list of these classes, see its help page using
\code{help(GsvaGeneSets)}.}

\item{\code{assay}}{Character vector of length 1.  The name of the assay to use in
case \code{exprData} is a multi-assay container, otherwise ignored.  By default,
the first assay is used.}

\item{\code{annotation}}{An object of class \code{GeneIdentifierType} from package
\code{GSEABase} describing the gene identifiers used as the row names of the
expression data set.  See \code{GeneIdentifierType} for help on available gene
identifier types and how to construct them.  This
information can be used to map gene identifiers occurring in the gene sets.
By default, this slot has value \code{NullIdentifier} and gene identifiers used in
expression data set and gene sets are matched directly.}

\item{\code{minSize}}{Numeric vector of length 1.  Minimum size of the resulting gene
sets after gene identifier mapping. By default, the minimum size is 1.}

\item{\code{maxSize}}{Numeric vector of length 1.  Maximum size of the resulting gene
sets after gene identifier mapping. By default, the maximum size is \code{Inf}.}

\item{\code{nzcount}}{Numeric vector of length 1. Number of non-zero values in the
selected assay, if there is more than one, of the 'exprData' slot.}

\item{\code{ondisk}}{Character vector of length 1 denoting whether an on-disk backend
should be used to reduce the memory footprint. The default value
\code{ondisk="auto"} will attempt to load all the data in main memory when the
input nonzero values fit in main memory, otherwise it will attempt working
with an on-disk data structure that reduces de memory footprint. When
\code{ondisk="yes"} it will attempt to work with an on-disk data structure, while
when \code{ondisk="no"} it will attempt to load all the data in main memory.}
}}

\examples{

library(GSVA)

p <- 10 ## number of genes
n <- 30 ## number of samples

gsets <- list(set1=paste0("g", 1:3),
              set2=paste0("g", 4:6),
              set3=paste0("g", 7:10),
              set4=paste0("g", 10:13)) ## genes not in the expression data
gsets

y <- matrix(rnorm(n*p), nrow=p, ncol=n,
            dimnames=list(paste("g", 1:p, sep="") , paste("s", 1:n, sep="")))

gsvapar <- gsvaParam(y, gsets)

class(gsvapar)
is(gsvapar, "GsvaMethodParam")

}
\seealso{
\code{\linkS4class{GsvaExprData}},
\code{\linkS4class{GsvaGeneSets}},
\code{\linkS4class{zscoreParam}},
\code{\linkS4class{plageParam}},
\code{\linkS4class{ssgseaParam}},
\code{\linkS4class{gsvaParam}},
\code{\link[GSEABase:GeneIdentifierType-class]{GeneIdentifierType}}

\code{\link{plageParam}}, \code{\link{zscoreParam}}, \code{\link{ssgseaParam}}, \code{\link{gsvaParam}}
}
