% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsva.R
\name{gsva-defunct}
\alias{gsva-defunct}
\title{Gene Set Variation Analysis}
\description{
This is the old manual page of the defunct version
of the function \code{gsva()}.
}
\seealso{
\code{\link{GSVA-pkg-defunct}}
}
\keyword{internal}
