% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsva.R
\name{gsva-deprecated}
\alias{gsva-deprecated}
\title{Gene Set Variation Analysis}
\usage{
gsva(expr, gset.idx.list, ...)
}
\description{
This is the old manual page of the deprecated version
of the function \code{gsva()}.
}
\seealso{
\code{\link{GSVA-pkg-deprecated}}
}
\keyword{internal}
