% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{GsvaExprData-class}
\alias{GsvaExprData-class}
\alias{GsvaExprData}
\title{\code{GsvaExprData} class}
\description{
Virtual superclass of expression data classes supported by \code{GSVA}.
}
\details{
\code{GSVA} supports expression data matrices in a growing number of containers
and representations.  This class union allows to store any of these in a slot
of another class as well as defining common methods for all of them.
}
\seealso{
\code{\link{matrix}},
\code{\link[Matrix:dgCMatrix-class]{dgCMatrix}},
\code{\link[SparseArray:SVT_SparseArray-class]{SVT_SparseMatrix}},
\code{\link[DelayedArray:DelayedArray-class]{DelayedMatrix}},
\code{\link[Biobase]{ExpressionSet}},
\code{\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}},
\code{\link[SingleCellExperiment:SingleCellExperiment]{SingleCellExperiment}},
\code{\link[SpatialExperiment:SpatialExperiment]{SpatialExperiment}},
\code{\link[HDF5Array:HDF5Array-class]{HDF5Array}}
}
